@@ -3,11 +3,15 @@ from __future__ import absolute_import
 from typing import Tuple, Union, Optional
 
 import h5py as h5
-
 from keras.utils import Sequence
 from keras.utils import to_categorical
+
+from albumentations import Compose
 import numpy as np
 
+available_modes = {'train', 'test'}
+available_labels_encoding = {'hot', 'smooth', False}
+    
 class HDF5ImageGenerator(Sequence):
     """Just a simple custom Keras HDF5 ImageDataGenerator.
     
@@ -19,9 +23,6 @@ class HDF5ImageGenerator(Sequence):
     ---------
     src : str
         Path of the hdf5 source file.
-    num_classes : int
-        Total number of classes.
-        Default is 2.
     X_key : str
         Key of the h5 file image tensors dataset.
         Default is "images".
@@ -84,7 +85,6 @@ class HDF5ImageGenerator(Sequence):
     """
     def __init__(self,
                  src,
-                 num_classes=2,
                  X_key='images',
                  y_key='labels',
                  batch_size=32,
@@ -95,17 +95,15 @@ class HDF5ImageGenerator(Sequence):
                  augmenter=False,
                  mode='train'):
         
-        if mode not in {'train', 'test'}:
+        if mode not in available_modes:
             raise ValueError(
                 '`mode` should be `"train"`'
                 '(fit_generator() and evaluate_generator()) or'
                 '`"test"` (predict_generator().'
                 'Received: %s' % mode)
         self.mode = mode
-        
-        print('Mode:', self.mode)
-        
-        if labels_encoding not in {'hot', 'smooth', False}:
+                
+        if labels_encoding not in available_labels_encoding:
             raise ValueError(
                 '`labels_encoding` should be `"hot"` '
                 '(classic binary matrix) or '
@@ -114,20 +112,27 @@ class HDF5ImageGenerator(Sequence):
                 'Received: %s' % labels_encoding)
         self.labels_encoding = labels_encoding
         
-        if self.labels_encoding == 'smooth' and not (0 < smooth_factor <= 1):
-            raise ValueError('`"smooth"` labels encoding'
-                             'must use a `"smooth_factor"`'
-                             '< 0 smooth_factor <= 1')
+        if (self.labels_encoding == 'smooth') and not (0 < smooth_factor <= 1):
+            raise ValueError(
+                '`"smooth"` labels encoding'
+                'must use a `"smooth_factor"`'
+                '< 0 smooth_factor <= 1')
         
+        if augmenter and not isinstance(augmenter, Compose):
+             raise ValueError(
+                 '`augmenter` argument'
+                 'must be an instance of albumentations'
+                 '`Compose` class.'
+                 'Received type: %s' % type(augmenter))
+        self.augmenter = augmenter
+            
         self.src: str = src
-        self.num_classes: int = num_classes
         self.X_key: str = X_key
         self.y_key: str = y_key
         self.batch_size: int = batch_size
         self.shuffle: bool = shuffle
         self.scaler: bool = scaler
         self.smooth_factor: float = smooth_factor
-        self.augmenter: Any = augmenter
         
         self.indices = np.arange(self.__get_dataset_shape(self.X_key, 0))
         
@@ -158,9 +163,10 @@ class HDF5ImageGenerator(Sequence):
         ---------
         indices : ndarray, 
             The list of current batch indices.
-        dataset : str or None
+        dataset : (optional) str
             The dataset key. If None, returns
             a batch of (image tensors, labels).
+            Defaults to None.
          
         Returns
         -------
@@ -207,7 +213,7 @@ class HDF5ImageGenerator(Sequence):
 
     def apply_labels_encoding(self,
                               batch_y: np.ndarray,
-                              smooth_factor: float = 0.0) -> np.ndarray:
+                              smooth_factor: Optional[float] = None) -> np.ndarray:
         """Converts a class vector (integers) to binary class matrix.
          See Keras to_categorical utils function.
          
@@ -215,17 +221,18 @@ class HDF5ImageGenerator(Sequence):
         ---------
         batch_y : np.ndarray
             Current batch integer labels.
-        smooth_factor : Int or Float
+        smooth_factor : (optional) Float
             Smooth factor.
+            Defaults to None.
         
         Returns
         -------
         np.ndarray
             A binary class matrix.
         """
-        batch_y = to_categorical(batch_y, num_classes=self.num_classes)
+        batch_y = to_categorical(batch_y)
         
-        if smooth_factor > 0:
+        if smooth_factor is not None:
             batch_y = self.apply_labels_smoothing(batch_y, factor=smooth_factor)
             
         return batch_y
@@ -299,11 +306,10 @@ class HDF5ImageGenerator(Sequence):
             
         # Shall we apply labels encoding?
         if self.labels_encoding:
-            batch_y = self.apply_labels_encoding(
-                batch_y,
+            batch_y = self.apply_labels_encoding(batch_y,
                 smooth_factor = self.smooth_factor
                     if self.labels_encoding == 'smooth'
-                    else 0)
+                    else None)
                                         
         return (batch_X, batch_y)
 
@@ -330,13 +336,16 @@ class HDF5ImageGenerator(Sequence):
             'test' : self.__next_batch_test
         }[self.mode](indices)
     
+    def __shuffle_indices(self):
+        """If the shuffle parameter is set to True,
+         dataset will be shuffled (in-place).
+         (not available in test 'mode').
+        """
+        if (self.mode == 'train') and self.shuffle:
+            np.random.shuffle(self.indices)
+    
     def on_epoch_end(self):
         """Triggered once at the very beginning as well as 
          at the end of each epoch.
-         
-         If the shuffle parameter is set to True,
-         image tensor indices will be shuffled.
-         (not available in test 'mode').
         """
-        if self.mode is 'train' and self.shuffle:
-            np.random.shuffle(self.indices)
\ No newline at end of file
+        self.__shuffle_indices()
\ No newline at end of file
